import { Driver } from "../driver/Driver";
import { Repository } from "../repository/Repository";
import { ObjectType } from "../common/ObjectType";
import { EntityManager } from "../entity-manager/EntityManager";
import { EntityMetadataCollection } from "../metadata-args/collection/EntityMetadataCollection";
import { ReactiveRepository } from "../repository/ReactiveRepository";
import { ReactiveEntityManager } from "../entity-manager/ReactiveEntityManager";
import { TreeRepository } from "../repository/TreeRepository";
import { TreeReactiveRepository } from "../repository/TreeReactiveRepository";
import { EntitySchema } from "../metadata/entity-schema/EntitySchema";
import { Broadcaster } from "../subscriber/Broadcaster";
import { SpecificRepository } from "../repository/SpecificRepository";
import { SpecificReactiveRepository } from "../repository/ReactiveSpecificRepository";
import { EntityMetadata } from "../metadata/EntityMetadata";
/**
 * A single connection instance to the database.
 * Each connection has its own entity manager, repositories, broadcaster and entity metadatas.
 */
export declare class Connection {
    /**
     * Connection name.
     */
    readonly name: string;
    /**
     * Database driver used by this connection.
     */
    readonly driver: Driver;
    /**
     * All entity metadatas that are registered for this connection.
     */
    readonly entityMetadatas: EntityMetadataCollection;
    /**
     * Used to broadcast connection events.
     */
    readonly broadcaster: Broadcaster;
    /**
     * Gets EntityManager of this connection.
     */
    private readonly _entityManager;
    /**
     * Gets ReactiveEntityManager of this connection.
     */
    private readonly _reactiveEntityManager;
    /**
     * Stores all registered metadatas with their repositories.
     */
    private readonly repositoryForMetadatas;
    /**
     * Entity listeners that are registered for this connection.
     */
    private readonly entityListeners;
    /**
     * Entity subscribers that are registered for this connection.
     */
    private readonly entitySubscribers;
    /**
     * Registered entity classes to be used for this connection.
     */
    private readonly entityClasses;
    /**
     * Registered entity schemas to be used for this connection.
     */
    private readonly entitySchemas;
    /**
     * Registered subscriber classes to be used for this connection.
     */
    private readonly subscriberClasses;
    /**
     * Registered naming strategy classes to be used for this connection.
     */
    private readonly namingStrategyClasses;
    /**
     * Naming strategy to be used in this connection.
     */
    private usedNamingStrategy;
    /**
     * Indicates if connection has been done or not.
     */
    private _isConnected;
    constructor(name: string, driver: Driver);
    /**
     * Indicates if connection to the database already established for this connection.
     */
    readonly isConnected: boolean;
    /**
     * Gets entity manager that allows to perform repository operations with any entity of this connection.
     */
    readonly entityManager: EntityManager;
    /**
     * Gets entity manager that allows to perform repository operations with any entity of this connection.
     * This version of entity manager is reactive - works with Observables instead of Promises.
     */
    readonly reactiveEntityManager: ReactiveEntityManager;
    /**
     * Performs connection to the database.
     */
    connect(): Promise<this>;
    /**
     * Closes connection with the database.
     * Once connection is closed, you cannot use repositories and perform any operations except
     * opening connection again.
     */
    close(): Promise<void>;
    /**
     * Drops the database and all its data.
     */
    dropDatabase(): Promise<void>;
    /**
     * Creates database schema for all entities registered in this connection.
     *
     * @param dropBeforeSync If set to true then it drops the database with all its tables and data
     * @param forceSync If set to true then schema creation will not be executed in a transaction
     */
    syncSchema(dropBeforeSync?: boolean, forceSync?: boolean): Promise<void>;
    /**
     * Imports entities from the given paths (directories) and registers them in the current connection.
     */
    importEntitiesFromDirectories(paths: string[]): Promise<this>;
    /**
     * Imports entity schemas from the given paths (directories) and registers them in the current connection.
     */
    importEntitySchemaFromDirectories(paths: string[]): Promise<this>;
    /**
     * Imports subscribers from the given paths (directories) and registers them in the current connection.
     */
    importSubscribersFromDirectories(paths: string[]): Promise<this>;
    /**
     * Imports naming strategies from the given paths (directories) and registers them in the current connection.
     */
    importNamingStrategiesFromDirectories(paths: string[]): Promise<this>;
    /**
     * Imports entities and registers them in the current connection.
     */
    importEntities(entities: Function[]): Promise<this>;
    /**
     * Imports schemas and registers them in the current connection.
     */
    importEntitySchemas(schemas: EntitySchema[]): Promise<this>;
    /**
     * Imports subscribers and registers them in the current connection.
     */
    importSubscribers(subscriberClasses: Function[]): Promise<this>;
    /**
     * Imports naming strategies and registers them in the current connection.
     */
    importNamingStrategies(strategies: Function[]): Promise<this>;
    /**
     * Sets given naming strategy to be used.
     * Naming strategy must be set to be used before connection is established.
     */
    useNamingStrategy(name: string): this;
    /**
     * Sets given naming strategy to be used.
     * Naming strategy must be set to be used before connection is established.
     */
    useNamingStrategy(strategy: Function): this;
    /**
     * Gets the entity metadata of the given entity class.
     */
    getMetadata(entity: Function): EntityMetadata;
    /**
     * Gets the entity metadata of the given entity name.
     */
    getMetadata(entity: string): EntityMetadata;
    /**
     * Gets repository for the given entity class.
     */
    getRepository<Entity>(entityClass: ObjectType<Entity>): Repository<Entity>;
    /**
     * Gets repository for the given entity name.
     */
    getRepository<Entity>(entityName: string): Repository<Entity>;
    /**
     * Gets tree repository for the given entity class.
     * Only tree-type entities can have a TreeRepository,
     * like ones decorated with @ClosureTable decorator.
     */
    getTreeRepository<Entity>(entityClass: ObjectType<Entity>): TreeRepository<Entity>;
    /**
     * Gets tree repository for the given entity class.
     * Only tree-type entities can have a TreeRepository,
     * like ones decorated with @ClosureTable decorator.
     */
    getTreeRepository<Entity>(entityName: string): TreeRepository<Entity>;
    /**
     * Gets specific repository for the given entity class.
     * SpecificRepository is a special repository that contains specific and non standard repository methods.
     */
    getSpecificRepository<Entity>(entityClass: ObjectType<Entity>): SpecificRepository<Entity>;
    /**
     * Gets specific repository for the given entity name.
     * SpecificRepository is a special repository that contains specific and non standard repository methods.
     */
    getSpecificRepository<Entity>(entityName: string): SpecificRepository<Entity>;
    /**
     * Gets reactive repository for the given entity class.
     * Reactive repositories has same functionality as regular repositories,
     * the only difference is that reactive repository methods return Observable instead of Promise.
     */
    getReactiveRepository<Entity>(entityClass: ObjectType<Entity>): ReactiveRepository<Entity>;
    /**
     * Gets reactive repository for the given entity name.
     * Reactive repositories has same functionality as regular repositories,
     * the only difference is that reactive repository methods return Observable instead of Promise.
     */
    getReactiveRepository<Entity>(entityName: string): ReactiveRepository<Entity>;
    /**
     * Gets reactive tree repository for the given entity class.
     * Only tree-type entities can have a TreeRepository,
     * like ones decorated with @ClosureTable decorator.
     * Reactive repositories has same functionality as regular repositories,
     * the only difference is that reactive repository methods return Observable instead of Promise.
     */
    getReactiveTreeRepository<Entity>(entityClass: ObjectType<Entity>): TreeReactiveRepository<Entity>;
    /**
     * Gets reactive tree repository for the given entity name.
     * Only tree-type entities can have a TreeRepository,
     * like ones decorated with @ClosureTable decorator.
     * Reactive repositories has same functionality as regular repositories,
     * the only difference is that reactive repository methods return Observable instead of Promise.
     */
    getReactiveTreeRepository<Entity>(entityName: string): TreeReactiveRepository<Entity>;
    /**
     * Gets specific repository for the given entity class.
     * SpecificReactiveRepository is a special repository that contains specific and non standard repository methods.
     * Reactive repositories has same functionality as regular repositories,
     * the only difference is that reactive repository methods return Observable instead of Promise.
     */
    getSpecificReactiveRepository<Entity>(entityClass: ObjectType<Entity>): SpecificReactiveRepository<Entity>;
    /**
     * Gets specific repository for the given entity name.
     * SpecificReactiveRepository is a special repository that contains specific and non standard repository methods.
     * Reactive repositories has same functionality as regular repositories,
     * the only difference is that reactive repository methods return Observable instead of Promise.
     */
    getSpecificReactiveRepository<Entity>(entityName: string): SpecificReactiveRepository<Entity>;
    /**
     * Finds repository with metadata of the given entity class or name.
     */
    private findRepositoryForMetadata(entityClassOrName, options?);
    /**
     * Builds all registered metadatas.
     */
    private buildMetadatas();
    /**
     * Creates a naming strategy to be used for this connection.
     */
    private createNamingStrategy();
}
