"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const RepositoryNotFoundError_1 = require("./error/RepositoryNotFoundError");
const EntityListenerMetadata_1 = require("../metadata/EntityListenerMetadata");
const DirectoryExportedClassesLoader_1 = require("../util/DirectoryExportedClassesLoader");
const index_1 = require("../index");
const EntityMetadataBuilder_1 = require("../metadata-builder/EntityMetadataBuilder");
const DefaultNamingStrategy_1 = require("../naming-strategy/DefaultNamingStrategy");
const EntityMetadataCollection_1 = require("../metadata-args/collection/EntityMetadataCollection");
const NoConnectionForRepositoryError_1 = require("./error/NoConnectionForRepositoryError");
const CannotImportAlreadyConnectedError_1 = require("./error/CannotImportAlreadyConnectedError");
const CannotCloseNotConnectedError_1 = require("./error/CannotCloseNotConnectedError");
const CannotConnectAlreadyConnectedError_1 = require("./error/CannotConnectAlreadyConnectedError");
const NamingStrategyNotFoundError_1 = require("./error/NamingStrategyNotFoundError");
const EntityManagerFactory_1 = require("../entity-manager/EntityManagerFactory");
const SchemaCreatorFactory_1 = require("../schema-creator/SchemaCreatorFactory");
const RepositoryNotTreeError_1 = require("./error/RepositoryNotTreeError");
const CannotSyncNotConnectedError_1 = require("./error/CannotSyncNotConnectedError");
const CannotUseNamingStrategyNotConnectedError_1 = require("./error/CannotUseNamingStrategyNotConnectedError");
const BroadcasterFactory_1 = require("../subscriber/BroadcasterFactory");
const CannotGetEntityManagerNotConnectedError_1 = require("./error/CannotGetEntityManagerNotConnectedError");
const LazyRelationsWrapper_1 = require("../repository/LazyRelationsWrapper");
const RepositoryForMetadata_1 = require("../repository/RepositoryForMetadata");
/**
 * A single connection instance to the database.
 * Each connection has its own entity manager, repositories, broadcaster and entity metadatas.
 */
class Connection {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    constructor(name, driver) {
        /**
         * All entity metadatas that are registered for this connection.
         */
        this.entityMetadatas = new EntityMetadataCollection_1.EntityMetadataCollection();
        /**
         * Stores all registered metadatas with their repositories.
         */
        this.repositoryForMetadatas = [];
        /**
         * Entity listeners that are registered for this connection.
         */
        this.entityListeners = [];
        /**
         * Entity subscribers that are registered for this connection.
         */
        this.entitySubscribers = [];
        /**
         * Registered entity classes to be used for this connection.
         */
        this.entityClasses = [];
        /**
         * Registered entity schemas to be used for this connection.
         */
        this.entitySchemas = [];
        /**
         * Registered subscriber classes to be used for this connection.
         */
        this.subscriberClasses = [];
        /**
         * Registered naming strategy classes to be used for this connection.
         */
        this.namingStrategyClasses = [];
        /**
         * Indicates if connection has been done or not.
         */
        this._isConnected = false;
        this.name = name;
        this.driver = driver;
        this._entityManager = index_1.getFromContainer(EntityManagerFactory_1.EntityManagerFactory).createEntityManager(this);
        this._reactiveEntityManager = index_1.getFromContainer(EntityManagerFactory_1.EntityManagerFactory).createReactiveEntityManager(this);
        this.broadcaster = index_1.getFromContainer(BroadcasterFactory_1.BroadcasterFactory).createBroadcaster(this.entityMetadatas, this.entitySubscribers, this.entityListeners);
    }
    // -------------------------------------------------------------------------
    // Accessors
    // -------------------------------------------------------------------------
    /**
     * Indicates if connection to the database already established for this connection.
     */
    get isConnected() {
        return this._isConnected;
    }
    /**
     * Gets entity manager that allows to perform repository operations with any entity of this connection.
     */
    get entityManager() {
        if (!this.isConnected)
            throw new CannotGetEntityManagerNotConnectedError_1.CannotGetEntityManagerNotConnectedError(this.name);
        return this._entityManager;
    }
    /**
     * Gets entity manager that allows to perform repository operations with any entity of this connection.
     * This version of entity manager is reactive - works with Observables instead of Promises.
     */
    get reactiveEntityManager() {
        if (!this.isConnected)
            throw new CannotGetEntityManagerNotConnectedError_1.CannotGetEntityManagerNotConnectedError(this.name);
        return this._reactiveEntityManager;
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Performs connection to the database.
     */
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isConnected)
                throw new CannotConnectAlreadyConnectedError_1.CannotConnectAlreadyConnectedError(this.name);
            // connect to the database via its driver
            yield this.driver.connect();
            // build all metadatas registered in the current connection
            this.buildMetadatas();
            // set connected status for the current connection
            this._isConnected = true;
            return this;
        });
    }
    /**
     * Closes connection with the database.
     * Once connection is closed, you cannot use repositories and perform any operations except
     * opening connection again.
     */
    close() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.isConnected)
                throw new CannotCloseNotConnectedError_1.CannotCloseNotConnectedError(this.name);
            yield this.driver.disconnect();
            this._isConnected = false;
        });
    }
    /**
     * Drops the database and all its data.
     */
    dropDatabase() {
        return __awaiter(this, void 0, void 0, function* () {
            const dbConnection = yield this.driver.retrieveDatabaseConnection();
            yield this.driver.clearDatabase(dbConnection);
            yield this.driver.releaseDatabaseConnection(dbConnection);
        });
    }
    /**
     * Creates database schema for all entities registered in this connection.
     *
     * @param dropBeforeSync If set to true then it drops the database with all its tables and data
     * @param forceSync If set to true then schema creation will not be executed in a transaction
     */
    syncSchema(dropBeforeSync = false, forceSync = false) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.isConnected)
                return Promise.reject(new CannotSyncNotConnectedError_1.CannotSyncNotConnectedError(this.name));
            const dbConnection = yield this.driver.retrieveDatabaseConnection();
            if (forceSync)
                yield this.driver.beginTransaction(dbConnection);
            try {
                if (dropBeforeSync)
                    yield this.driver.clearDatabase(dbConnection);
                const schemaBuilder = this.driver.createSchemaBuilder(dbConnection);
                const schemaCreatorFactory = index_1.getFromContainer(SchemaCreatorFactory_1.SchemaCreatorFactory);
                const schemaCreator = schemaCreatorFactory.create(schemaBuilder, this.entityMetadatas);
                yield schemaCreator.create();
                yield this.driver.releaseDatabaseConnection(dbConnection);
                if (forceSync)
                    yield this.driver.commitTransaction(dbConnection);
            }
            catch (error) {
                if (forceSync)
                    yield this.driver.rollbackTransaction(dbConnection);
                throw error;
            }
        });
    }
    /**
     * Imports entities from the given paths (directories) and registers them in the current connection.
     */
    importEntitiesFromDirectories(paths) {
        return __awaiter(this, void 0, void 0, function* () {
            this.importEntities(DirectoryExportedClassesLoader_1.importClassesFromDirectories(paths));
            return this;
        });
    }
    /**
     * Imports entity schemas from the given paths (directories) and registers them in the current connection.
     */
    importEntitySchemaFromDirectories(paths) {
        return __awaiter(this, void 0, void 0, function* () {
            this.importEntitySchemas(DirectoryExportedClassesLoader_1.importJsonsFromDirectories(paths));
            return this;
        });
    }
    /**
     * Imports subscribers from the given paths (directories) and registers them in the current connection.
     */
    importSubscribersFromDirectories(paths) {
        return __awaiter(this, void 0, void 0, function* () {
            this.importSubscribers(DirectoryExportedClassesLoader_1.importClassesFromDirectories(paths));
            return this;
        });
    }
    /**
     * Imports naming strategies from the given paths (directories) and registers them in the current connection.
     */
    importNamingStrategiesFromDirectories(paths) {
        return __awaiter(this, void 0, void 0, function* () {
            this.importEntities(DirectoryExportedClassesLoader_1.importClassesFromDirectories(paths));
            return this;
        });
    }
    /**
     * Imports entities and registers them in the current connection.
     */
    importEntities(entities) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isConnected)
                throw new CannotImportAlreadyConnectedError_1.CannotImportAlreadyConnectedError("entities", this.name);
            entities.forEach(cls => this.entityClasses.push(cls));
            return this;
        });
    }
    /**
     * Imports schemas and registers them in the current connection.
     */
    importEntitySchemas(schemas) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isConnected)
                throw new CannotImportAlreadyConnectedError_1.CannotImportAlreadyConnectedError("schemas", this.name);
            schemas.forEach(schema => this.entitySchemas.push(schema));
            return this;
        });
    }
    /**
     * Imports subscribers and registers them in the current connection.
     */
    importSubscribers(subscriberClasses) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isConnected)
                throw new CannotImportAlreadyConnectedError_1.CannotImportAlreadyConnectedError("entity subscribers", this.name);
            subscriberClasses.forEach(cls => this.subscriberClasses.push(cls));
            return this;
        });
    }
    /**
     * Imports naming strategies and registers them in the current connection.
     */
    importNamingStrategies(strategies) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isConnected)
                throw new CannotImportAlreadyConnectedError_1.CannotImportAlreadyConnectedError("naming strategies", this.name);
            strategies.forEach(cls => this.namingStrategyClasses.push(cls));
            return this;
        });
    }
    /**
     * Sets given naming strategy to be used.
     * Naming strategy must be set to be used before connection is established.
     */
    useNamingStrategy(strategyClassOrName) {
        if (this.isConnected)
            throw new CannotUseNamingStrategyNotConnectedError_1.CannotUseNamingStrategyNotConnectedError(this.name);
        this.usedNamingStrategy = strategyClassOrName;
        return this;
    }
    /**
     Gets entity metadata for the given entity class or schema name.
     */
    getMetadata(entity) {
        return this.entityMetadatas.findByTarget(entity);
    }
    /**
     * Gets repository for the given entity class or name.
     */
    getRepository(entityClassOrName) {
        const repositoryForMetadata = this.findRepositoryForMetadata(entityClassOrName);
        return repositoryForMetadata.repository;
    }
    /**
     * Gets tree repository for the given entity class or name.
     * Only tree-type entities can have a TreeRepository,
     * like ones decorated with @ClosureTable decorator.
     */
    getTreeRepository(entityClassOrName) {
        const repositoryForMetadata = this.findRepositoryForMetadata(entityClassOrName, { checkIfTreeTable: true });
        return repositoryForMetadata.repository;
    }
    /**
     * Gets specific repository for the given entity class or name.
     * SpecificRepository is a special repository that contains specific and non standard repository methods.
     */
    getSpecificRepository(entityClassOrName) {
        const repositoryForMetadata = this.findRepositoryForMetadata(entityClassOrName);
        return repositoryForMetadata.specificRepository;
    }
    /**
     * Gets reactive repository for the given entity class or name.
     * Reactive repositories has same functionality as regular repositories,
     * the only difference is that reactive repository methods return Observable instead of Promise.
     */
    getReactiveRepository(entityClassOrName) {
        const repositoryForMetadata = this.findRepositoryForMetadata(entityClassOrName);
        return repositoryForMetadata.reactiveRepository;
    }
    /**
     * Gets reactive tree repository for the given entity class or name.
     * Only tree-type entities can have a TreeRepository,
     * like ones decorated with @ClosureTable decorator.
     * Reactive repositories has same functionality as regular repositories,
     * the only difference is that reactive repository methods return Observable instead of Promise.
     */
    getReactiveTreeRepository(entityClassOrName) {
        const repositoryForMetadata = this.findRepositoryForMetadata(entityClassOrName, { checkIfTreeTable: true });
        return repositoryForMetadata.reactiveRepository;
    }
    /**
     * Gets specific repository for the given entity class or name.
     * SpecificReactiveRepository is a special repository that contains specific and non standard repository methods.
     * Reactive repositories has same functionality as regular repositories,
     * the only difference is that reactive repository methods return Observable instead of Promise.
     */
    getSpecificReactiveRepository(entityClassOrName) {
        const repositoryForMetadata = this.findRepositoryForMetadata(entityClassOrName);
        return repositoryForMetadata.specificReactiveRepository;
    }
    // -------------------------------------------------------------------------
    // Private Methods
    // -------------------------------------------------------------------------
    /**
     * Finds repository with metadata of the given entity class or name.
     */
    findRepositoryForMetadata(entityClassOrName, options) {
        if (!this.isConnected)
            throw new NoConnectionForRepositoryError_1.NoConnectionForRepositoryError(this.name);
        if (!this.entityMetadatas.hasTarget(entityClassOrName))
            throw new RepositoryNotFoundError_1.RepositoryNotFoundError(this.name, entityClassOrName);
        const metadata = this.entityMetadatas.findByTarget(entityClassOrName);
        const repositoryForMetadata = this.repositoryForMetadatas.find(metadataRepository => metadataRepository.metadata === metadata);
        if (!repositoryForMetadata)
            throw new RepositoryNotFoundError_1.RepositoryNotFoundError(this.name, entityClassOrName);
        if (options && options.checkIfTreeTable && !metadata.table.isClosure)
            throw new RepositoryNotTreeError_1.RepositoryNotTreeError(entityClassOrName);
        return repositoryForMetadata;
    }
    /**
     * Builds all registered metadatas.
     */
    buildMetadatas() {
        this.entitySubscribers.length = 0;
        this.entityListeners.length = 0;
        this.repositoryForMetadatas.length = 0;
        this.entityMetadatas.length = 0;
        const namingStrategy = this.createNamingStrategy();
        const lazyRelationsWrapper = new LazyRelationsWrapper_1.LazyRelationsWrapper(this.driver, this.entityMetadatas, this.broadcaster);
        // take imported event subscribers
        if (this.subscriberClasses && this.subscriberClasses.length) {
            index_1.getMetadataArgsStorage()
                .entitySubscribers
                .filterByTargets(this.subscriberClasses)
                .map(metadata => index_1.getFromContainer(metadata.target))
                .forEach(subscriber => this.entitySubscribers.push(subscriber));
        }
        // take imported entity listeners
        if (this.entityClasses && this.entityClasses.length) {
            index_1.getMetadataArgsStorage()
                .entityListeners
                .filterByTargets(this.entityClasses)
                .forEach(metadata => this.entityListeners.push(new EntityListenerMetadata_1.EntityListenerMetadata(metadata)));
        }
        // build entity metadatas from metadata args storage (collected from decorators)
        if (this.entityClasses && this.entityClasses.length) {
            index_1.getFromContainer(EntityMetadataBuilder_1.EntityMetadataBuilder)
                .buildFromMetadataArgsStorage(lazyRelationsWrapper, namingStrategy, this.entityClasses)
                .forEach(metadata => {
                this.entityMetadatas.push(metadata);
                this.repositoryForMetadatas.push(new RepositoryForMetadata_1.RepositoryForMetadata(this, metadata));
            });
        }
        // build entity metadatas from given entity schemas
        if (this.entitySchemas && this.entitySchemas.length) {
            index_1.getFromContainer(EntityMetadataBuilder_1.EntityMetadataBuilder)
                .buildFromSchemas(lazyRelationsWrapper, namingStrategy, this.entitySchemas)
                .forEach(metadata => {
                this.entityMetadatas.push(metadata);
                this.repositoryForMetadatas.push(new RepositoryForMetadata_1.RepositoryForMetadata(this, metadata));
            });
        }
    }
    /**
     * Creates a naming strategy to be used for this connection.
     */
    createNamingStrategy() {
        // if naming strategies are not loaded, or used naming strategy is not set then use default naming strategy
        if (!this.namingStrategyClasses || !this.namingStrategyClasses.length || !this.usedNamingStrategy)
            return index_1.getFromContainer(DefaultNamingStrategy_1.DefaultNamingStrategy);
        // try to find used naming strategy in the list of loaded naming strategies
        const namingMetadata = index_1.getMetadataArgsStorage()
            .namingStrategies
            .filterByTargets(this.namingStrategyClasses)
            .find(strategy => {
            if (typeof this.usedNamingStrategy === "string") {
                return strategy.name === this.usedNamingStrategy;
            }
            else {
                return strategy.target === this.usedNamingStrategy;
            }
        });
        // throw an error if not found
        if (!namingMetadata)
            throw new NamingStrategyNotFoundError_1.NamingStrategyNotFoundError(this.usedNamingStrategy, this.name);
        // initialize a naming strategy instance
        return index_1.getFromContainer(namingMetadata.target);
    }
}
exports.Connection = Connection;

//# sourceMappingURL=Connection.js.map
