"use strict";
/**
 * Common functions shared between different manager types.
 */
class BaseEntityManager {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    constructor(connection) {
        this.connection = connection;
    }
    /**
     * Gets repository for the given entity class or name.
     */
    getRepository(entityClassOrName) {
        return this.obtainRepository(entityClassOrName);
    }
    /**
     * Gets tree repository for the given entity class or name.
     */
    getTreeRepository(entityClassOrName) {
        return this.obtainTreeRepository(entityClassOrName);
    }
    /**
     * Gets reactive repository of the given entity.
     */
    getReactiveRepository(entityClass) {
        return this.obtainReactiveRepository(entityClass);
    }
    /**
     * Gets specific repository of the given entity.
     */
    getSpecificRepository(entityClass) {
        return this.obtainSpecificRepository(entityClass);
    }
    /**
     * Gets specific reactive repository of the given entity.
     */
    getSpecificReactiveRepository(entityClass) {
        return this.obtainSpecificReactiveRepository(entityClass);
    }
    /**
     * Gets reactive tree repository of the given entity.
     */
    getTreeReactiveRepository(entityClass) {
        return this.obtainTreeReactiveRepository(entityClass);
    }
    /**
     * Checks if entity has an id by its Function type or schema name.
     */
    hasId(targetOrEntity, maybeEntity) {
        const target = arguments.length === 2 ? targetOrEntity : targetOrEntity.constructor;
        const entity = arguments.length === 2 ? maybeEntity : targetOrEntity;
        return this.obtainRepository(target).hasId(entity);
    }
    /**
     * Creates a new query builder that can be used to build an sql query.
     */
    createQueryBuilder(entityClass, alias) {
        return this.obtainRepository(entityClass).createQueryBuilder(alias);
    }
    /**
     * Creates a new entity instance or instances.
     * Can copy properties from the given object into new entities.
     */
    create(entityClass, plainObjectOrObjects) {
        if (plainObjectOrObjects instanceof Array) {
            return this.obtainRepository(entityClass).create(plainObjectOrObjects);
        }
        else if (plainObjectOrObjects) {
            return this.obtainRepository(entityClass).create(plainObjectOrObjects);
        }
        else {
            return this.obtainRepository(entityClass).create();
        }
    }
    /**
     * Creates a new entity from the given plan javascript object. If entity already exist in the database, then
     * it loads it (and everything related to it), replaces all values with the new ones from the given object
     * and returns this new entity. This new entity is actually a loaded from the db entity with all properties
     * replaced from the new object.
     */
    preload(entityClass, object) {
        return this.obtainRepository(entityClass).preload(object);
    }
    /**
     * Merges two entities into one new entity.
     */
    merge(entityClass, ...objects) {
        return this.obtainRepository(entityClass).merge(...objects);
    }
    // -------------------------------------------------------------------------
    // Private Methods
    // -------------------------------------------------------------------------
    /**
     * We are using a private function to avoid type problems, because we are sending a Function everywhere in the
     * code, and obtainRepository does not accept a Function, and only ObjectType it can accept - it brings
     * us type problems. To avoid this we are using private function here.
     */
    obtainRepository(entityClassOrName) {
        if (typeof entityClassOrName === "string") {
            return this.connection.getRepository(entityClassOrName);
        }
        else {
            return this.connection.getRepository(entityClassOrName);
        }
    }
    /**
     * We are using a private function to avoid type problems, because we are sending a Function everywhere in the
     * code, and obtainTreeRepository does not accept a Function, and only ObjectType it can accept - it brings
     * us type problems. To avoid this we are using private function here.
     */
    obtainTreeRepository(entityClassOrName) {
        if (typeof entityClassOrName === "string") {
            return this.connection.getTreeRepository(entityClassOrName);
        }
        else {
            return this.connection.getTreeRepository(entityClassOrName);
        }
    }
    /**
     * We are using a private function to avoid type problems, because we are sending a Function everywhere in the
     * code, and obtainTreeRepository does not accept a Function, and only ObjectType it can accept - it brings
     * us type problems. To avoid this we are using private function here.
     */
    obtainSpecificRepository(entityClassOrName) {
        if (typeof entityClassOrName === "string") {
            return this.connection.getSpecificRepository(entityClassOrName);
        }
        else {
            return this.connection.getSpecificRepository(entityClassOrName);
        }
    }
    /**
     * We are using a private function to avoid type problems, because we are sending a Function everywhere in the
     * code, and obtainReactiveRepository does not accept a Function, and only ObjectType it can accept - it brings
     * us type problems. To avoid this we are using private function here.
     */
    obtainReactiveRepository(entityClassOrName) {
        if (typeof entityClassOrName === "string") {
            return this.connection.getReactiveRepository(entityClassOrName);
        }
        else {
            return this.connection.getReactiveRepository(entityClassOrName);
        }
    }
    /**
     * We are using a private function to avoid type problems, because we are sending a Function everywhere in the
     * code, and obtainReactiveTreeRepository does not accept a Function, and only ObjectType it can accept - it brings
     * us type problems. To avoid this we are using private function here.
     */
    obtainTreeReactiveRepository(entityClassOrName) {
        if (typeof entityClassOrName === "string") {
            return this.connection.getReactiveTreeRepository(entityClassOrName);
        }
        else {
            return this.connection.getReactiveTreeRepository(entityClassOrName);
        }
    }
    /**
     * We are using a private function to avoid type problems, because we are sending a Function everywhere in the
     * code, and obtainReactiveRepository does not accept a Function, and only ObjectType it can accept - it brings
     * us type problems. To avoid this we are using private function here.
     */
    obtainSpecificReactiveRepository(entityClassOrName) {
        if (typeof entityClassOrName === "string") {
            return this.connection.getSpecificReactiveRepository(entityClassOrName);
        }
        else {
            return this.connection.getSpecificReactiveRepository(entityClassOrName);
        }
    }
}
exports.BaseEntityManager = BaseEntityManager;

//# sourceMappingURL=BaseEntityManager.js.map
