"use strict";
const Rx = require("rxjs/Rx");
const BaseEntityManager_1 = require("./BaseEntityManager");
/**
 * Entity manager supposed to work with any entity, automatically find its repository and call its method, whatever
 * entity type are you passing. This version of ReactiveEntityManager works with reactive streams and observables.
 */
class ReactiveEntityManager extends BaseEntityManager_1.BaseEntityManager {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    constructor(connection) {
        super(connection);
    }
    persist(targetOrEntity, maybeEntity) {
        // todo: extra casting is used strange tsc error here, check later maybe typescript bug
        const target = arguments.length === 2 ? targetOrEntity : targetOrEntity.constructor;
        const entity = arguments.length === 2 ? maybeEntity : targetOrEntity;
        return this.getReactiveRepository(target).persist(entity);
    }
    remove(targetOrEntity, maybeEntity) {
        // todo: extra casting is used strange tsc error here, check later maybe typescript bug
        const target = arguments.length === 2 ? targetOrEntity : targetOrEntity.constructor;
        const entity = arguments.length === 2 ? maybeEntity : targetOrEntity;
        return this.getReactiveRepository(target).remove(entity);
    }
    /**
     * Finds entities that match given conditions.
     */
    find(entityClass, conditionsOrFindOptions, options) {
        if (conditionsOrFindOptions && options) {
            return this.getReactiveRepository(entityClass).find(conditionsOrFindOptions, options);
        }
        else if (conditionsOrFindOptions) {
            return this.getReactiveRepository(entityClass).find(conditionsOrFindOptions);
        }
        else {
            return this.getReactiveRepository(entityClass).find();
        }
    }
    /**
     * Finds entities that match given conditions.
     */
    findAndCount(entityClass, conditionsOrFindOptions, options) {
        if (conditionsOrFindOptions && options) {
            return this.getReactiveRepository(entityClass).findAndCount(conditionsOrFindOptions, options);
        }
        else if (conditionsOrFindOptions) {
            return this.getReactiveRepository(entityClass).findAndCount(conditionsOrFindOptions);
        }
        else {
            return this.getReactiveRepository(entityClass).findAndCount();
        }
    }
    /**
     * Finds first entity that matches given conditions.
     */
    findOne(entityClass, conditionsOrFindOptions, options) {
        if (conditionsOrFindOptions && options) {
            return this.getReactiveRepository(entityClass).findOne(conditionsOrFindOptions, options);
        }
        else if (conditionsOrFindOptions) {
            return this.getReactiveRepository(entityClass).findOne(conditionsOrFindOptions);
        }
        else {
            return this.getReactiveRepository(entityClass).findOne();
        }
    }
    /**
     * Finds entity with given id.
     */
    findOneById(entityClass, id, options) {
        return this.getReactiveRepository(entityClass).findOneById(id, options);
    }
    /**
     * Executes raw SQL query and returns raw database results.
     */
    query(query) {
        const promise = this.connection.driver
            .retrieveDatabaseConnection()
            .then(dbConnection => {
            return this.connection.driver.query(dbConnection, query)
                .then(result => {
                return this.connection.driver
                    .releaseDatabaseConnection(dbConnection)
                    .then(() => result);
            });
        });
        return Rx.Observable.fromPromise(promise);
    }
    /**
     * Wraps given function execution (and all operations made there) in a transaction.
     */
    transaction(runInTransaction) {
        const promise = this.connection.driver
            .retrieveDatabaseConnection()
            .then(dbConnection => {
            let runInTransactionResult;
            return this.connection.driver
                .beginTransaction(dbConnection)
                .then(() => runInTransaction())
                .then(result => {
                runInTransactionResult = result;
                return this.connection.driver
                    .commitTransaction(dbConnection)
                    .then(() => this.connection.driver.releaseDatabaseConnection(dbConnection))
                    .then(() => runInTransactionResult);
            });
        });
        return Rx.Observable.fromPromise(promise);
    }
}
exports.ReactiveEntityManager = ReactiveEntityManager;

//# sourceMappingURL=ReactiveEntityManager.js.map
