import { Connection } from "../connection/Connection";
import { EntityMetadata } from "../metadata/EntityMetadata";
import { QueryBuilder } from "../query-builder/QueryBuilder";
import { PlainObjectToNewEntityTransformer } from "../query-builder/transformer/PlainObjectToNewEntityTransformer";
import { PlainObjectToDatabaseEntityTransformer } from "../query-builder/transformer/PlainObjectToDatabaseEntityTransformer";
import { EntityPersistOperationBuilder } from "../persistment/EntityPersistOperationsBuilder";
import { PersistOperationExecutor } from "../persistment/PersistOperationExecutor";
import { FindOptions } from "./FindOptions";
import { ObjectLiteral } from "../common/ObjectLiteral";
/**
 * Repository is supposed to work with your entity objects. Find entities, insert, update, delete, etc.
 */
export declare class Repository<Entity extends ObjectLiteral> {
    protected connection: Connection;
    protected metadata: EntityMetadata;
    protected persistOperationExecutor: PersistOperationExecutor;
    protected entityPersistOperationBuilder: EntityPersistOperationBuilder;
    protected plainObjectToEntityTransformer: PlainObjectToNewEntityTransformer;
    protected plainObjectToDatabaseEntityTransformer: PlainObjectToDatabaseEntityTransformer<Entity>;
    constructor(connection: Connection, metadata: EntityMetadata);
    /**
     * Returns object that is managed by this repository.
     * If this repository manages entity from schema, then it returns a name of that schema instead.
     */
    readonly target: Function | string;
    /**
     * Checks if entity has an id.
     */
    hasId(entity: Entity): boolean;
    /**
     * Creates a new query builder that can be used to build a sql query.
     */
    createQueryBuilder(alias: string): QueryBuilder<Entity>;
    /**
     * Creates a new entity instance.
     */
    create(): Entity;
    /**
     * Creates a new entities and copies all entity properties from given objects into their new entities.
     * Note that it copies only properties that present in entity schema.
     */
    create(plainObjects: Object[]): Entity[];
    /**
     * Creates a new entity instance and copies all entity properties from this object into a new entity.
     * Note that it copies only properties that present in entity schema.
     */
    create(plainObject: Object): Entity;
    /**
     * Creates a new entity from the given plan javascript object. If entity already exist in the database, then
     * it loads it (and everything related to it), replaces all values with the new ones from the given object
     * and returns this new entity. This new entity is actually a loaded from the db entity with all properties
     * replaced from the new object.
     */
    preload(object: Object): Promise<Entity>;
    /**
     * Merges multiple entities (or entity-like objects) into one new entity.
     */
    merge(...objects: ObjectLiteral[]): Entity;
    /**
     * Persists (saves) all given entities in the database.
     * If entities do not exist in the database then inserts, otherwise updates.
     */
    persist(entities: Entity[]): Promise<Entity[]>;
    /**
     * Persists (saves) a given entity in the database.
     * If entity does not exist in the database then inserts, otherwise updates.
     */
    persist(entity: Entity): Promise<Entity>;
    /**
     * Removes a given entities from the database.
     */
    remove(entities: Entity[]): Promise<Entity[]>;
    /**
     * Removes a given entity from the database.
     */
    remove(entity: Entity): Promise<Entity>;
    /**
     * Finds all entities.
     */
    find(): Promise<Entity[]>;
    /**
     * Finds entities that match given conditions.
     */
    find(conditions: Object): Promise<Entity[]>;
    /**
     * Finds entities with given find options.
     */
    find(options: FindOptions): Promise<Entity[]>;
    /**
     * Finds entities that match given conditions and find options.
     */
    find(conditions: Object, options: FindOptions): Promise<Entity[]>;
    /**
     * Finds entities that match given conditions.
     */
    findAndCount(): Promise<[Entity[], number]>;
    /**
     * Finds entities that match given conditions.
     */
    findAndCount(conditions: Object): Promise<[Entity[], number]>;
    /**
     * Finds entities that match given conditions.
     */
    findAndCount(options: FindOptions): Promise<[Entity[], number]>;
    /**
     * Finds entities that match given conditions.
     */
    findAndCount(conditions: Object, options: FindOptions): Promise<[Entity[], number]>;
    /**
     * Finds first entity that matches given conditions.
     */
    findOne(): Promise<Entity>;
    /**
     * Finds first entity that matches given conditions.
     */
    findOne(conditions: Object): Promise<Entity>;
    /**
     * Finds first entity that matches given find options.
     */
    findOne(options: FindOptions): Promise<Entity>;
    /**
     * Finds first entity that matches given conditions and find options.
     */
    findOne(conditions: Object, options: FindOptions): Promise<Entity>;
    /**
     * Finds entity with given id.
     */
    findOneById(id: any, options?: FindOptions): Promise<Entity>;
    /**
     * Executes a raw SQL query and returns a raw database results.
     */
    query(query: string): Promise<any>;
    /**
     * Wraps given function execution (and all operations made there) in a transaction.
     */
    transaction(runInTransaction: () => Promise<any> | any): Promise<any>;
    private createFindQueryBuilder(conditionsOrFindOptions?, options?);
    /**
     * When ORM loads dbEntity it uses joins to load all entity dependencies. However when dbEntity is newly persisted
     * to the db, but uses already exist in the db relational entities, those entities cannot be loaded, and will
     * absent in dbEntities. To fix it, we need to go throw all persistedEntities we have, find out those which have
     * ids, check if we did not load them yet and try to load them. This algorithm will make sure that all dbEntities
     * are loaded. Further it will help insert operations to work correctly.
     */
    private findNotLoadedIds(dbConnection, dbEntities, persistedEntities);
    /**
     * Extracts unique objects from given entity and all its downside relations.
     */
    private extractObjectsById(entity, metadata, entityWithIds?);
}
