"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const QueryBuilder_1 = require("../query-builder/QueryBuilder");
const PlainObjectToNewEntityTransformer_1 = require("../query-builder/transformer/PlainObjectToNewEntityTransformer");
const PlainObjectToDatabaseEntityTransformer_1 = require("../query-builder/transformer/PlainObjectToDatabaseEntityTransformer");
const EntityPersistOperationsBuilder_1 = require("../persistment/EntityPersistOperationsBuilder");
const PersistOperationExecutor_1 = require("../persistment/PersistOperationExecutor");
const FindOptions_1 = require("./FindOptions");
/**
 * Repository is supposed to work with your entity objects. Find entities, insert, update, delete, etc.
 */
class Repository {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    constructor(connection, metadata) {
        this.connection = connection;
        this.metadata = metadata;
        this.entityPersistOperationBuilder = new EntityPersistOperationsBuilder_1.EntityPersistOperationBuilder(connection.entityMetadatas);
        this.plainObjectToEntityTransformer = new PlainObjectToNewEntityTransformer_1.PlainObjectToNewEntityTransformer();
        this.plainObjectToDatabaseEntityTransformer = new PlainObjectToDatabaseEntityTransformer_1.PlainObjectToDatabaseEntityTransformer();
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Returns object that is managed by this repository.
     * If this repository manages entity from schema, then it returns a name of that schema instead.
     */
    get target() {
        return this.metadata.target;
    }
    /**
     * Checks if entity has an id.
     */
    hasId(entity) {
        const columnName = this.metadata.primaryColumn.propertyName;
        return !!entity &&
            entity.hasOwnProperty(columnName) &&
            entity[columnName] !== null &&
            entity[columnName] !== undefined &&
            entity[columnName] !== "";
    }
    /**
     * Creates a new query builder that can be used to build a sql query.
     */
    createQueryBuilder(alias) {
        return new QueryBuilder_1.QueryBuilder(this.connection.driver, this.connection.entityMetadatas, this.connection.broadcaster /*, dbConnection*/) // todo: better to pass connection?
            .select(alias)
            .from(this.metadata.target, alias);
    }
    /**
     * Creates a new entity instance or instances.
     * Can copy properties from the given object into new entities.
     */
    create(plainObjectOrObjects) {
        if (plainObjectOrObjects instanceof Array) {
            return plainObjectOrObjects.map(object => this.create(object));
        }
        const newEntity = this.metadata.create();
        if (plainObjectOrObjects)
            this.plainObjectToEntityTransformer.transform(newEntity, plainObjectOrObjects, this.metadata);
        return newEntity;
    }
    /**
     * Creates a new entity from the given plan javascript object. If entity already exist in the database, then
     * it loads it (and everything related to it), replaces all values with the new ones from the given object
     * and returns this new entity. This new entity is actually a loaded from the db entity with all properties
     * replaced from the new object.
     */
    preload(object) {
        const queryBuilder = this.createQueryBuilder(this.metadata.table.name);
        return this.plainObjectToDatabaseEntityTransformer.transform(object, this.metadata, queryBuilder);
    }
    /**
     * Merges multiple entities (or entity-like objects) into one new entity.
     */
    merge(...objects) {
        const newEntity = this.create();
        objects.forEach(object => this.plainObjectToEntityTransformer.transform(newEntity, object, this.metadata));
        return newEntity;
    }
    /**
     * Persists one or many given entities.
     */
    persist(entityOrEntities) {
        return __awaiter(this, void 0, void 0, function* () {
            // if multiple entities given then go throw all of them and save them
            if (entityOrEntities instanceof Array)
                return Promise.all(entityOrEntities.map(entity => this.persist(entity)));
            const dbConnection = yield this.connection.driver.retrieveDatabaseConnection();
            const allPersistedEntities = yield this.extractObjectsById(entityOrEntities, this.metadata);
            let loadedDbEntity = null;
            if (this.hasId(entityOrEntities)) {
                const queryBuilder = new QueryBuilder_1.QueryBuilder(this.connection.driver, this.connection.entityMetadatas, this.connection.broadcaster, dbConnection) // todo: better to pass connection?
                    .select(this.metadata.table.name)
                    .from(this.metadata.target, this.metadata.table.name);
                loadedDbEntity = yield this.plainObjectToDatabaseEntityTransformer.transform(entityOrEntities, this.metadata, queryBuilder);
            }
            let entityWithIds = [];
            if (loadedDbEntity)
                entityWithIds = yield this.extractObjectsById(loadedDbEntity, this.metadata);
            // need to find db entities that were not loaded by initialize method
            const allDbEntities = yield this.findNotLoadedIds(dbConnection, entityWithIds, allPersistedEntities);
            const persistedEntity = {
                id: this.metadata.getEntityId(entityOrEntities),
                entityTarget: this.metadata.target,
                entity: entityOrEntities
            };
            const dbEntity = {
                id: this.metadata.getEntityId(loadedDbEntity),
                entityTarget: this.metadata.target,
                entity: loadedDbEntity
            };
            const persistOperation = this.entityPersistOperationBuilder.buildFullPersistment(this.metadata, dbEntity, persistedEntity, allDbEntities, allPersistedEntities);
            const persistOperationExecutor = new PersistOperationExecutor_1.PersistOperationExecutor(this.connection.driver, this.connection.entityMetadatas, this.connection.broadcaster, dbConnection); // todo: better to pass connection?
            yield persistOperationExecutor.executePersistOperation(persistOperation);
            yield this.connection.driver.releaseDatabaseConnection(dbConnection);
            return entityOrEntities;
        });
    }
    /**
     * Removes one or many given entities.
     */
    remove(entityOrEntities) {
        return __awaiter(this, void 0, void 0, function* () {
            // if multiple entities given then go throw all of them and save them
            if (entityOrEntities instanceof Array)
                return Promise.all(entityOrEntities.map(entity => this.remove(entity)));
            const dbConnection = yield this.connection.driver.retrieveDatabaseConnection();
            const queryBuilder = new QueryBuilder_1.QueryBuilder(this.connection.driver, this.connection.entityMetadatas, this.connection.broadcaster, dbConnection) // todo: better to pass connection?
                .select(this.metadata.table.name)
                .from(this.metadata.target, this.metadata.table.name);
            const dbEntity = yield this.plainObjectToDatabaseEntityTransformer.transform(entityOrEntities, this.metadata, queryBuilder);
            entityOrEntities[this.metadata.primaryColumn.name] = undefined;
            const [dbEntities, allPersistedEntities] = yield Promise.all([
                this.extractObjectsById(dbEntity, this.metadata),
                this.extractObjectsById(entityOrEntities, this.metadata)
            ]);
            const entityWithId = {
                id: this.metadata.getEntityId(entityOrEntities),
                entityTarget: this.metadata.target,
                entity: entityOrEntities
            };
            const dbEntityWithId = {
                id: this.metadata.getEntityId(dbEntity),
                entityTarget: this.metadata.target,
                entity: dbEntity
            };
            const persistOperation = this.entityPersistOperationBuilder.buildOnlyRemovement(this.metadata, dbEntityWithId, entityWithId, dbEntities, allPersistedEntities);
            const persistOperationExecutor = new PersistOperationExecutor_1.PersistOperationExecutor(this.connection.driver, this.connection.entityMetadatas, this.connection.broadcaster, dbConnection); // todo: better to pass connection?
            yield persistOperationExecutor.executePersistOperation(persistOperation);
            yield this.connection.driver.releaseDatabaseConnection(dbConnection);
            return entityOrEntities;
        });
    }
    /**
     * Finds entities that match given conditions and/or find options.
     */
    find(conditionsOrFindOptions, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.createFindQueryBuilder(conditionsOrFindOptions, options)
                .getResults();
        });
    }
    /**
     * Finds entities that match given conditions.
     */
    findAndCount(conditionsOrFindOptions, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.createFindQueryBuilder(conditionsOrFindOptions, options)
                .getResultsAndCount();
        });
    }
    /**
     * Finds first entity that matches given conditions and/or find options.
     */
    findOne(conditionsOrFindOptions, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.createFindQueryBuilder(conditionsOrFindOptions, options)
                .getSingleResult();
        });
    }
    /**
     * Finds entity with given id.
     */
    findOneById(id, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.createFindQueryBuilder({ [this.metadata.primaryColumn.name]: id }, options)
                .getSingleResult();
        });
    }
    /**
     * Executes a raw SQL query and returns a raw database results.
     */
    query(query) {
        return __awaiter(this, void 0, void 0, function* () {
            const databaseConnection = yield this.connection.driver.retrieveDatabaseConnection();
            const result = yield this.connection.driver.query(databaseConnection, query);
            yield this.connection.driver.releaseDatabaseConnection(databaseConnection);
            return result;
        });
    }
    /**
     * Wraps given function execution (and all operations made there) in a transaction.
     */
    transaction(runInTransaction) {
        return __awaiter(this, void 0, void 0, function* () {
            let runInTransactionResult;
            const databaseConnection = yield this.connection.driver.retrieveDatabaseConnection();
            return this.connection.driver
                .beginTransaction(databaseConnection)
                .then(() => runInTransaction())
                .then(result => {
                runInTransactionResult = result;
                return this.connection.driver.commitTransaction(databaseConnection);
            })
                .catch(err => {
                return this.connection.driver.rollbackTransaction(databaseConnection)
                    .then(() => {
                    throw err;
                })
                    .catch(() => {
                    throw err;
                });
            })
                .then(() => this.connection.driver.releaseDatabaseConnection(databaseConnection))
                .then(() => runInTransactionResult);
        });
    }
    // -------------------------------------------------------------------------
    // Private Methods
    // -------------------------------------------------------------------------
    createFindQueryBuilder(conditionsOrFindOptions, options) {
        const findOptions = FindOptions_1.FindOptionsUtils.isFindOptions(conditionsOrFindOptions) ? conditionsOrFindOptions : options;
        const conditions = FindOptions_1.FindOptionsUtils.isFindOptions(conditionsOrFindOptions) ? undefined : conditionsOrFindOptions;
        const alias = findOptions ? findOptions.alias : this.metadata.table.name;
        const qb = this.createQueryBuilder(alias);
        if (findOptions) {
            FindOptions_1.FindOptionsUtils.applyOptionsToQueryBuilder(qb, findOptions);
        }
        if (conditions) {
            Object.keys(conditions).forEach(key => {
                const name = key.indexOf(".") === -1 ? alias + "." + key : key;
                qb.andWhere(name + "=:" + key);
            });
            qb.addParameters(conditions);
        }
        return qb;
    }
    /**
     * When ORM loads dbEntity it uses joins to load all entity dependencies. However when dbEntity is newly persisted
     * to the db, but uses already exist in the db relational entities, those entities cannot be loaded, and will
     * absent in dbEntities. To fix it, we need to go throw all persistedEntities we have, find out those which have
     * ids, check if we did not load them yet and try to load them. This algorithm will make sure that all dbEntities
     * are loaded. Further it will help insert operations to work correctly.
     */
    findNotLoadedIds(dbConnection, dbEntities, persistedEntities) {
        const missingDbEntitiesLoad = persistedEntities
            .filter(entityWithId => entityWithId.id !== null && entityWithId.id !== undefined)
            .filter(entityWithId => !dbEntities.find(dbEntity => dbEntity.entityTarget === entityWithId.entityTarget && dbEntity.id === entityWithId.id))
            .map(entityWithId => {
            const metadata = this.connection.entityMetadatas.findByTarget(entityWithId.entityTarget);
            const alias = entityWithId.entityTarget.constructor.name;
            const qb = new QueryBuilder_1.QueryBuilder(this.connection.driver, this.connection.entityMetadatas, this.connection.broadcaster, dbConnection)
                .select(alias)
                .from(entityWithId.entityTarget, alias)
                .where(alias + "." + this.metadata.primaryColumn.propertyName + "=:id", { id: entityWithId.id })
                .getSingleResult();
            // const repository = this.connection.getRepository(entityWithId.entityTarget as any); // todo: fix type
            return qb.then(loadedEntity => {
                if (!loadedEntity)
                    return undefined;
                return {
                    id: loadedEntity[metadata.primaryColumn.name],
                    entityTarget: metadata.target,
                    entity: loadedEntity
                };
            });
        });
        return Promise.all(missingDbEntitiesLoad).then(missingDbEntities => {
            return dbEntities.concat(missingDbEntities.filter(dbEntity => !!dbEntity));
        });
    }
    /**
     * Extracts unique objects from given entity and all its downside relations.
     */
    extractObjectsById(entity, metadata, entityWithIds = []) {
        const promises = metadata.relations.map(relation => {
            const relMetadata = relation.inverseEntityMetadata;
            const value = relation.isLazy ? entity["__" + relation.propertyName + "__"] : entity[relation.propertyName];
            if (!value)
                return undefined;
            if (value instanceof Array) {
                const subPromises = value.map((subEntity) => {
                    return this.extractObjectsById(subEntity, relMetadata, entityWithIds);
                });
                return Promise.all(subPromises);
            }
            else {
                return this.extractObjectsById(value, relMetadata, entityWithIds);
            }
        });
        return Promise.all(promises.filter(result => !!result)).then(() => {
            if (!entityWithIds.find(entityWithId => entityWithId.entity === entity)) {
                entityWithIds.push({
                    id: entity[metadata.primaryColumn.name],
                    entityTarget: metadata.target,
                    entity: entity
                });
            }
            return entityWithIds;
        });
    }
}
exports.Repository = Repository;

//# sourceMappingURL=Repository.js.map
